@@ -30,6 +30,10 @@ test_that("PostgreSQL", {
     "data_64_bit_numeric_warning", # TODO
     "data_64_bit_lossless", # TODO
     "data_integer", # Fails, unreliably on Windows
+    # TODO: fails with Pro Driver with "a statement is already in progress"
+    "send_query_immediate",
+    # TODO: fails with Pro Driver with "a statement is already in progress"
+    "send_query_only_one_result_set",
     "send_query_syntax_error", # TODO
     "get_query_syntax_error", # TODO
     "send_query_params", # TODO
@@ -53,7 +57,6 @@ test_that("PostgreSQL", {
     "unquote_identifier_vectorized", # TODO
     "create_table_overwrite", # TODO
     "create_table_error", # TODO
-    "create_temporary_table", # TODO
     "append_table_.*", # TODO
     "append_roundtrip_.*", # TODO
     "append_table_.*", # TODO
