@@ -81,6 +81,32 @@ test_that("parse_database_error() works with messages from the wild", {
           [Microsoft][ODBC Driver 18 for SQL Server]TCP Provider: Error code 0x2726
           [Microsoft][ODBC Driver 18 for SQL Server]A network-related or instance-specific error has occurred while establishing a connection to 127.0.0.1. Server is not found or not accessible. Check if instance name is correct and if SQL Server is configured to allow remote connections. For more information see SQL Server Books Online. "
   expect_snapshot(error = TRUE, rethrow_database_error(msg, call = NULL))
+
+  # message contains brackets that may be interpreted as inline markup (#859)
+  msg <- "nanodbc/nanodbc.cpp:1722: S1000
+          [SAP AG][LIBODBCHDB DLL][HDBODBC] General error;403 internal error: Error opening the cursor for the remote database <***.***> [SAP AG][LIBODBCHDB SO][HDBODBC] Connection not open;-10807 Connection down: [89013] Socket closed by peer {***.**.*.**:***** -> ***.**.***.**:***** TenantName:(none) SiteVolumeID:1:3 SiteType:PRIMARY ConnectionID:****** SessionID:************}
+          <SQL> 'SELECT DISTINCT \"po_id\", ***CENSORED***'"
+  expect_snapshot(error = TRUE, rethrow_database_error(msg, call = NULL))
+
+  msg <- "`parse_database_error()` will not {be able to parse this}, but it should still be successfully rethrown as-is."
+  expect_snapshot(error = TRUE, rethrow_database_error(msg, call = NULL))
+})
+
+test_that("set_database_error_names() works (#840)", {
+  expect_equal(
+    set_database_error_names(c("unnamed", "vector")),
+    c(x = "unnamed", `*` = "vector")
+  )
+  expect_equal(set_database_error_names("unnamed scalar"), c(x = "unnamed scalar"))
+  expect_equal(
+    set_database_error_names(c("i" = "partially", "named")),
+    c(i = "partially", `*` = "named")
+  )
+  expect_equal(
+    set_database_error_names(c("partially", "i" = "named")),
+    c(x = "partially", i = "named")
+  )
+  expect_equal(set_database_error_names(c("i" = "named")), c(i = "named"))
 })
 
 test_that("set_odbcsysini() works (#791)", {
@@ -120,19 +146,29 @@ test_that("check_attributes()", {
   )
 })
 
-test_that("configure_spark() returns early on windows", {
+test_that("configure_simba() returns early on windows", {
   local_mocked_bindings(is_macos = function() {FALSE})
 
-  expect_equal(configure_spark(), NULL)
+  expect_equal(configure_simba(), NULL)
 })
 
-test_that("configure_spark() errors informatively on failure to install unixODBC", {
+test_that("configure_simba() errors informatively on failure to install unixODBC", {
   local_mocked_bindings(
     is_macos = function() {TRUE},
     locate_install_unixodbc = function() {character(0)}
   )
 
-  expect_snapshot(databricks(), error = TRUE)
+  expect_snapshot(configure_simba(), error = TRUE)
+})
+
+test_that("configure_simba() calls configure_unixodbc_simba() (#835)", {
+  skip_if_not(is_macos())
+  local_mocked_bindings(
+    locate_install_unixodbc = function() "hey",
+    configure_unixodbc_simba = function(...) TRUE
+  )
+
+  expect_true(configure_simba(list(path = "example")))
 })
 
 test_that("locate_install_unixodbc() returns reasonable values", {
@@ -148,18 +184,28 @@ test_that("locate_install_unixodbc() returns reasonable values", {
 test_that("databricks() errors informatively when spark ini isn't writeable", {
   local_mocked_bindings(is_writeable = function(path) {FALSE})
   expect_snapshot(
-    write_spark_lines("", ".", ".", call2("databricks")),
+    write_simba_lines("", ".", ".", call2("databricks")),
     error = TRUE
   )
 })
 
-test_that("locate_config_spark() returns reasonable values", {
-  simba_spark_ini <- "some/folder/simba.sparkodbc.ini"
-  withr::local_envvar(SIMBASPARKINI = simba_spark_ini)
-  expect_equal(locate_config_spark(), simba_spark_ini)
+test_that("driver_dir(...) returns reasonable values", {
+  path <- "/some/path/driver.so"
+  local_mocked_bindings(odbcListDrivers = function() {
+    data.frame(name = "OG Driver", attribute = "Driver",
+      value = path, drop = FALSE)
+  })
+  expect_equal(driver_dir("OG Driver"), "/some/path")
+  expect_equal(driver_dir(path), "/some/path")
 })
 
-test_that("configure_unixodbc_spark() writes reasonable entries", {
+test_that("spark_simba_config() returns reasonable values", {
+  simba_spark_ini <- "some/folder/simba.sparkodbc.ini"
+  withr::local_envvar(SIMBASPARKINI = simba_spark_ini)
+  expect_equal(spark_simba_config(""), simba_spark_ini)
+})
+
+test_that("configure_unixodbc_simba() writes reasonable entries", {
   unixodbc_install_path <- "libodbcinst.dylib"
   spark_config_path <- "simba.sparkodbc.ini"
 
@@ -171,9 +217,15 @@ test_that("configure_unixodbc_spark() writes reasonable entries", {
     con = spark_config_path
   )
 
-  configure_unixodbc_spark(
+  expect_snapshot(configure_unixodbc_simba(
     unixodbc_install = unixodbc_install_path,
-    spark_config = spark_config_path
+    simba_config = spark_config_path,
+    action = "warn"
+  ))
+  configure_unixodbc_simba(
+    unixodbc_install = unixodbc_install_path,
+    simba_config = spark_config_path,
+    action = "modify"
   )
 
   expect_equal(
@@ -186,7 +238,8 @@ test_that("configure_unixodbc_spark() writes reasonable entries", {
     )
   )
 
-  # both of the relevant fields are already there:
+  # both of the relevant fields are already there
+  # but point to incorrect values
   writeLines(
     c("some=entries",
       "not=relevant",
@@ -195,9 +248,48 @@ test_that("configure_unixodbc_spark() writes reasonable entries", {
     con = spark_config_path
   )
 
-  res <- configure_unixodbc_spark(
+  expect_snapshot(configure_unixodbc_simba(
     unixodbc_install = unixodbc_install_path,
-    spark_config = spark_config_path
+    simba_config = spark_config_path,
+    action = "warn"
+  ))
+  res <- configure_unixodbc_simba(
+    unixodbc_install = unixodbc_install_path,
+    simba_config = spark_config_path,
+    action = "modify"
+  )
+
+  expect_equal(res, NULL)
+  expect_equal(
+    readLines(spark_config_path),
+    c(
+      "some=entries",
+      "not=relevant",
+      "ODBCInstLib=libodbcinst.dylib",
+      "DriverManagerEncoding=UTF-16"
+    )
+  )
+
+  # One entry correct, other incorrect
+  # expect warning with single suggestion
+  # when action is "warn"
+  writeLines(
+    c("some=entries",
+      "not=relevant",
+      "ODBCInstLib=libodbcinst.dylib",
+      "DriverManagerEncoding=UTF-32"),
+    con = spark_config_path
+  )
+
+  expect_snapshot(configure_unixodbc_simba(
+    unixodbc_install = unixodbc_install_path,
+    simba_config = spark_config_path,
+    action = "warn"
+  ))
+  res <- configure_unixodbc_simba(
+    unixodbc_install = unixodbc_install_path,
+    simba_config = spark_config_path,
+    action = "modify"
   )
 
   expect_equal(res, NULL)
@@ -220,9 +312,15 @@ test_that("configure_unixodbc_spark() writes reasonable entries", {
     con = spark_config_path
   )
 
-  configure_unixodbc_spark(
+  expect_snapshot(configure_unixodbc_simba(
     unixodbc_install = unixodbc_install_path,
-    spark_config = spark_config_path
+    simba_config = spark_config_path,
+    action = "warn"
+  ))
+  configure_unixodbc_simba(
+    unixodbc_install = unixodbc_install_path,
+    simba_config = spark_config_path,
+    action = "modify"
   )
 
   expect_equal(
@@ -236,4 +334,19 @@ test_that("configure_unixodbc_spark() writes reasonable entries", {
       "DriverManagerEncoding=UTF-16"
     )
   )
+
+  # Finally, a good config
+  writeLines(
+    c("some=entries",
+      "not=relevant",
+      "ODBCInstLib=libodbcinst.dylib",
+      "DriverManagerEncoding=UTF-16"),
+    con = spark_config_path
+  )
+
+  expect_no_warning(configure_unixodbc_simba(
+    unixodbc_install = unixodbc_install_path,
+    simba_config = spark_config_path,
+    action = "warn"
+  ))
 })
